/*
 * Server che ritorna la lista di un direttorio remoto: versione con 
 * esecuzione ls.
 * 
 * Si lascia come utile esercizio la scrittura del programma che esegue ls
 * remoto utilizzando le opportune API (opendir, readdir, ecc.
 * 
 * Inviare eventuali soluzioni.
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#define DIM_DIR 80
/********************************************************/
void gestore(int signo)
{
  int stato;
  printf("esecuzione gestore di SIGCHLD\n");
  wait(&stato);
}


/********************************************************/

int main(int argc, char **argv)
{
  int  listen_sd, conn_sd;
  int port, len;
  char car1, car2, ccar;
  const int on = 1;
  struct sockaddr_in cliaddr, servaddr;
  struct hostent *host;

  /* CONTROLLO ARGOMENTI ---------------------------------- */
  if(argc!=2)
  {
    printf("Error: %s port\n", argv[0]);
    exit(1);
  }
  else port = atoi(argv[1]);

  /* INIZIALIZZAZIONE INDIRIZZO SERVER ----------------------------------------- */

  memset ((char *)&servaddr, 0, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_addr.s_addr = INADDR_ANY;
  servaddr.sin_port = htons(port);


  /* CREAZIONE E SETTAGGI SOCKET D'ASCOLTO --------------------------------------- */

  listen_sd=socket(AF_INET, SOCK_STREAM, 0);
  if(listen_sd <0)
  {perror("creazione socket "); exit(1);}
  printf("Server: creata la socket d'ascolto per le richieste di ordinamento, fd=%d\n", listen_sd);

  if(bind(listen_sd,(struct sockaddr *) &servaddr, sizeof(servaddr))<0)
  {perror("bind socket d'ascolto"); exit(1);}
  printf("Server: bind socket d'ascolto ok\n");

  if(setsockopt(listen_sd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))<0)
  {perror("set opzioni socket d'ascolto"); exit(1);}
  printf("Server: set opzioni socket d'ascolto ok\n");

  if (listen(listen_sd, 5)<0)
  {perror("listen"); exit(1);}
  printf("Server: listen ok\n");


  /* AGGANCIO GESTORE PER EVITARE FIGLI ZOMBIE,
   * Quali altre primitive potrei usare? E' portabile su tutti i sistemi?
   * Pregi/Difetti?
   * Alcune risposte le potete trovare nel materiale aggiuntivo!
   */
  signal(SIGCHLD, gestore);


  /* CICLO DI RICEZIONE RICHIESTE --------------------------------------------- */
  for(;;)
  {
  	len = sizeof(cliaddr);
    if((conn_sd=accept(listen_sd,(struct sockaddr *)&cliaddr,&len))<0)
    {
      if (errno==EINTR)
      {
        perror("Forzo la continuazione della accept");
        continue;
      }
      else exit(1);
    }


    if (fork()==0)
    { // figlio
      close(listen_sd);
      host=gethostbyaddr( (char *) &cliaddr.sin_addr, sizeof(cliaddr.sin_addr), AF_INET);
      if (host == NULL){ printf("client host information not found\n"); continue;  }
      else printf("Server (figlio): host client e' %s \n", host->h_name);

	  // Leggo la richiesta del client
	  if( (read(conn_sd, &car1, 1)>0) &&
          (read(conn_sd, &car2, 1)>0) ){
	    printf("Server (figlio), primo: %c e secondo: %c carattere\n", car1, car2);
      	// ridirezione dell'output del processo figlio e dello stderr
	  }
	  
	  //sostituzioni
	  while( read(conn_sd, &ccar, 1)>0 ) 
	    if( ccar==car1 ) write(conn_sd, &car2, 1);
	    else write(conn_sd, &ccar, 1);

	  // Libero risorse
	  close(conn_sd);

    } // figlio

    close(conn_sd);

  } // ciclo for infinito

}

